// BookTestCol.cpp: implementation of the BookTestCol class.
//
//////////////////////////////////////////////////////////////////////

#include "BookTestCol.h"
#include <DpfMl.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BookTestCol::BookTestCol()
{
   	m_PClock.Init("BookTestTb.BookTest.clock", AtPos);
	m_ResetN.Init("BookTestTb.BookTest.reset_n");
}

BookTestCol::~BookTestCol()
{

}

void BookTestCol::Start()
{
	m_ProcCtrls.push_back(StartTProc(this, (SPVPM)&BookTestCol::PacketInThread));
	m_ProcCtrls.push_back(StartTProc(this, (SPVPM)&BookTestCol::PacketOutThread));
	StartTProc(this, (SPVPM)&BookTestCol::ResetThread);
}

void BookTestCol::ResetThread()
{
    while(1)
    {
        Wait(m_ResetN, AtNeg);
        for(unsigned i = 0; i < m_ProcCtrls.size(); i++)
        {
            m_DataFifo.clear();
            m_ProcCtrls[i]->Restart();
        }
    }
}

void BookTestCol::PacketOutThread()
{
   	SpvSig data("BookTestTb.BookTest.data_out");
	SpvSig dataEn("BookTestTb.BookTest.data_out_valid");

    Wait(m_ResetN, AtPos);

    while(1)
    {
        Wait(m_PClock);
        if(dataEn == 1)
        {
            unsigned received = data.Uint();

            if(m_DataFifo.empty())
            {
                SPV_OUT(<<"["<<SimTime()<<"] Unexpected: Received, "<<received<<endl);
                SpvConfig::StopSim(true);  
                return;
            }
            
            unsigned input      = m_DataFifo.front();
            unsigned expected   = TransferFunc(input);
            if(!Compare(received, expected))
            {
                SPV_OUT(<<"["<<SimTime()<<"] Mismatch: Received, "<<received<<
                    ", Expected, "<<expected<<endl);
                SpvConfig::StopSim(true);
            }
            else
            {
                SPV_OUT(<<"["<<SimTime()<<"] Compare OK: Received, "<<received<<endl);            
            }
            
            m_DataFifo.pop_front();
        }
    }

}

void BookTestCol::PacketInThread()
{
   	SpvSig data("BookTestTb.BookTest.data_in");
	SpvSig dataEn("BookTestTb.BookTest.data_in_valid");

    Wait(m_ResetN, AtPos);

    while(1)
    {
        Wait(m_PClock);
        if(dataEn == 1)
            m_DataFifo.push_back(data.Uint());
    }
}

//Simulated MCC compiled function
void mlfGmFunc(mxArray** out, mxArray* in)
{
    MlStructArray inStruct(in);
    MlStructArray outStruct;

    //Prevent release of memory when outStruct goes out of scope.
    outStruct.SetNoDestroy(true);   

    //This is the actual transfer function "algorithm"
    unsigned dataIn = inStruct["DataIn"];
    outStruct["DataOut"] = ~dataIn & 0xF;

    //Set return value.  Important to do this at the end and 
    //not at initialization of outStruct
    *out = outStruct;
}

unsigned BookTestCol::TransferFunc(unsigned input)
{
    MlStructArray in, out;
    out.SetNoDestroy(true);

    in["DataIn"] = input;

    mlfGmFunc(out, in);

    unsigned retVal = out["DataOut"];
    return retVal;
}

/*
unsigned BookTestCol::TransferFunc(unsigned input)
{
    return (~input) & 0x7;
}
*/
bool BookTestCol::Compare(unsigned received, unsigned expected)
{
    if(received == expected)
        return true;
    else
        return false;
}

